/* constraint_manager.h
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#ifndef _CONSTRAINT_MANAGER_H_
#define _CONSTRAINT_MANAGER_H_

#include "ai.h"

#include "objects.h"
#include "constraints.h"
#include <list>

using namespace std;

class Constraint_Manager {
	//lista objektov, kjer se nahajajo vsi objekti
	list<Object*> *new_objects;
	list<Object*> *changed_objects;
	list<Object*> *unchanged_objects;
	list<Object*> *object_list;

	list<Object*> *group;

	list<Object*>::iterator iter_obj;
	//lista vseh omejitev
	list<Constraint*> *constraint_list;
	list<Constraint*>::iterator iter_cn;
	
	//nasa umetna inteligenca
	AI *ai;

	//doda konstraint
	void add_constraint(Constraint *c);

public:
	Object *obj_changed;
	//ustvarim managerja z obema listama
	Constraint_Manager(AI *_ai, list <Object*> *obj_list, list <Constraint*> *cn_list);
	
	//poskrbi za pravilno umestitev objekta
	//torej: ga prilagodi trenutnim objektov na sliki
	// in poveze z vsemi omejitvami
	void add_object(Object *o);

	//odstrani omejitev in vse z njo povezane 
	//strukture seznani s to odstranitvijo
	//lahko klicem z memberjem iz seznama ali omejitvjio samo

	//odstrani objekt in vse z njim povezane 
	//strukture seznani s to odstranitvjio
	//lahko klicem z memberjem iz seznama ali objektom samim
	void delete_object(Object *o);
	void delete_constraint(Constraint *c);


//IMPORTANTE!!!!!!!!
	int round_value(double value);
	
	int are_lines_parallel(Line *l1, Line *l2, float epsilon);
	int are_lines_equal_length(Line *fixed, Line *flex, float epsilon);
	int are_lines_connected(Line *fixed, Line *flex, float epsilon);
	int is_line_vertical(Line *line, float epsilon);
	int is_line_horizontal(Line *line, float epsilon);
	
	int make_line_horizontal(Line *line);
	int make_line_vertical(Line *line);
	int make_lines_parallel(Line *l1, Line *l2, int mode);
	int make_line_equal_length(Line *l1, Line *l2, int mode);
	int connect_lines(Line *l1, Line *l2, int i, int mode, int mode2);

	int check_for_constraint_type(Object *o, int type);
	

	void add_and_check(list<Object*> *objects, list<Object*> *new_objects);
	int check_constraints(Object *o);
	void recheck_objects(Object *o);
	void split_changed_objects();
	void recheck_constraints();

	unsigned char check_for_connection_with_older_line(Line *line, unsigned char vertex);
	unsigned char check_for_connection(Line *line, unsigned char vertex);

	void clear_visited_on_constraints();
	void clear_flags_on_objects();
	
	unsigned char move_vertex_by(Object *o, unsigned char vertex, Point *dp);
	int move_object_vertex_by(Object *o, int vertex, Point *dp);
	unsigned char move_object_by(Object *o, Point *dp);
	int move_object_whole_by(Object *o, Point *dp);

	void remove_broken_constraints();

	void set_connections_u(Line *line, int u);
	void remove_constraints_from_line(Line *line);

	Point *calc_DP(Point *Start, Point *End);
	void printout_point(Point *T);
	void printout_line(Line *l);
	double distance_between_points(Point *T1, Point *T2);

	void draw_objects(FXDCWindow *dc);
	void draw_constraints(FXDCWindow *dc);
	void straightener();

	//v listo group shrani vse objekte, ki so povezani z objektom o preko katerega koli objekta/objektov
	void group_connected_objects(list<Object*> *group, Object *o);
	//
	Object *find_closest_to_connect(Object *o, float epsilon);
};



#endif //_CONSTRAINT_MANAGER_H_