/* constraint_manager.cpp
 * Copyright (C) 2004 Djukic Marko, Pernus Klemen, Zagar Miha
 * For conditions of distribution and use, see the copyright notice in main.cpp
 */
#include "constraint_manager.h"

#include <iostream>


//konstruktor
Constraint_Manager::Constraint_Manager(AI *_ai, list <Object*> *obj_list, list <Constraint*> *cn_list) {
	ai = _ai;
	object_list = obj_list;
	constraint_list = cn_list;
	new_objects = new list <Object*>;
	changed_objects = new list <Object*>;
	unchanged_objects = new list <Object*>;

	group = new list <Object*>;

	obj_changed = NULL;
}

void Constraint_Manager::add_object(Object *o) {
	list <Object*>::iterator iter;
	list <Object*> *temp;
	temp = new list<Object*>;

	Object *closest;

	if (o != NULL) //dokler so, dodajamo nove objekte
	{
		new_objects->push_back(o);
	}
	else if (!new_objects->empty())
	{
		//preverimo nove objekte
		closest = find_closest_to_connect(new_objects->front(),10);
		if (closest != NULL)
		{
			group_connected_objects(group, closest);
			clear_flags_on_objects();
			clear_visited_on_constraints();
			for (iter = new_objects->begin(); iter != new_objects->end(); iter++)
			{
				temp->push_back((*iter));
				add_and_check(group,temp);
				group->push_back((*iter));
				temp->clear();
			}
		}
		else
			add_and_check(object_list, new_objects);

		cout << "stevilo novih objektov: " << new_objects->size() << endl;
		for (iter = new_objects->begin(); iter != new_objects->end(); iter++)
		{	
			object_list->push_back((*iter));
			check_constraints((*iter));
		}
	
		//remove_broken_constraints();
		new_objects->clear();
	}
}

void Constraint_Manager::add_constraint(Constraint *c) {
	//dodam omejitev v listo omejitev 
	constraint_list->push_back(c);
	c->add_to_objects();
}

void Constraint_Manager::delete_object(Object *o) {
	//list <Object*>::iterator iter_obj;
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;

	while (!o->constraints.empty())
	{
		delete_constraint(o->constraints.front());
		o->constraints.pop_front();
	}
	object_list->remove(o);
}

void Constraint_Manager::delete_constraint(Constraint *c) {
	constraint_list->remove(c);
	c->remove_from_objects();
}



//DELO Z OMEJITVAMI:
//funkcije za preverjanje omejitev
int Constraint_Manager::are_lines_parallel(Line *l1, Line *l2, float epsilon)
{
	Point *fixed_t1, *fixed_t2, *flex_t1, *flex_t2;
	Line *t1t1, *t2t2;
	
	fixed_t1 = new Point(l1->T1);
	fixed_t2 = new Point(l1->T2);

	flex_t1 = new Point(l2->T1);
	flex_t2 = new Point(l2->T2);

	float angle = abs(l1->get_angle() - l2->get_angle());

	if (angle > 90)
		angle = abs(angle - 180);

	if (angle > 30)
		return 0;

	if (distance_between_points(fixed_t1,flex_t1) + distance_between_points(fixed_t2,flex_t2) >
		distance_between_points(fixed_t1,flex_t2) + distance_between_points(fixed_t1,flex_t1))
	{
		flex_t1 = new Point(l2->T2);
		flex_t2 = new Point(l2->T1);
	}


	if (are_lines_equal_length(l1,l2,epsilon) == 1)
	{
		t1t1 = new Line(fixed_t1,flex_t1);
		t2t2 = new Line(fixed_t2,flex_t2);

		if (are_lines_equal_length(t1t1,t2t2,epsilon) == 1)
		{
			return 1;
		}
	}

	return 0;
}

int Constraint_Manager::are_lines_equal_length(Line *fixed, Line *flex, float epsilon)
{
	if (abs(fixed->get_length() - flex->get_length()) < epsilon)
		return 1;
	else
		return 0;
}

int Constraint_Manager::are_lines_connected(Line *fixed, Line *flex, float epsilon)
{
	int i, j;
	for (i = 1; i <= 2; i++)
	{
		for (j = 1; j <= 2; j++)
		{
			if (distance_between_points(fixed->get_vertex_point(i), flex->get_vertex_point(j)) < epsilon)
			{
				cout << "ugotovili smo, da sta crti " << fixed->get_age() << " in " << flex->get_age() << " staknjeni v ";
				cout << "T" << i << " in T" << j << endl;
				return ((i*2 - 2) + j);
			}
		}
	}
	return 0;
}

int Constraint_Manager::is_line_vertical(Line *line, float epsilon)
{
	double angle = line->get_angle();

	if (abs(abs(angle) - 90) < epsilon)
		return 1;
	else
		return 0;
}

int Constraint_Manager::is_line_horizontal(Line *line, float epsilon)
{
	double angle = line->get_angle();
	cout << "Angle: " << angle << endl;

	if (abs(angle) < epsilon)
		return 1;
	else
		return 0;
}

//popravek crt glede na omejitve
int Constraint_Manager::make_line_vertical(Line *line)
{
	int y1,y2;
	int x = line->get_center_of_line()->x;
	int y = line->get_center_of_line()->y;
	
	double length = line->get_length();


	if (line->T1->y < line->T2->y)
	{
		y1 = y - length/2;
		y2 = y + length/2;
	}
	else
	{
		y1 = y + length/2;
		y2 = y - length/2;
	}

	line->move_vertex_to(1,new Point(x,y1));
	line->move_vertex_to(2,new Point(x,y2));

	return 1;
}

int Constraint_Manager::make_line_horizontal(Line *line)
{
	int x1,x2;
	int x = line->get_center_of_line()->x;
	int y = line->get_center_of_line()->y;

	double length = line->get_length();

	if (line->T1->x < line->T2->x)
	{
		x1 = x - length/2;
		x2 = x + length/2;
	}
	else
	{
		x1 = x + length/2;
		x2 = x - length/2;
	}
	
	line->move_vertex_to(1,new Point(x1,y));
	line->move_vertex_to(2,new Point(x2,y));

	return 1;
}

int Constraint_Manager::make_lines_parallel(Line *l1, Line *l2, int mode)
{
Point *smerni_vektor_1, *smerni_vektor_2, *nova_tocka, *sredina_l1, *sredina_l2, *T1, *T2;
	Line *temp_line_1, *temp_line_2;
	float diff_1, diff_2;

	switch(mode)
	{
	case 0: //okrog sredine
		//
		cout << "delam poravnavo na nacin 0" << endl;
		sredina_l1 = new Point(l1->get_center_of_line());
		sredina_l2 = new Point(l2->get_center_of_line());
		smerni_vektor_1 = calc_DP(sredina_l1, sredina_l2);
		T1 = new Point(l1->T1);
		T1->move_by(smerni_vektor_1);
		T2 = new Point(l1->T2);
		T2->move_by(smerni_vektor_1);
		if (distance_between_points(l2->T1, T1) + distance_between_points(l2->T2, T2) <  
			distance_between_points(l2->T1, T2) + distance_between_points(l2->T2, T1))
		{
			l2->T1->move_to(T1);
			l2->T2->move_to(T2);
		}
		else
		{
			l2->T1->move_to(T2);
			l2->T2->move_to(T1);
		}
		break;
	case 1: //okrog T1
		//Prva moznost
		smerni_vektor_1 = calc_DP(l1->T1, l2->T1);
		T1 = new Point(l1->T1);
		T1->move_by(smerni_vektor_1);
		T2 = new Point(l1->T2);
		T2->move_by(smerni_vektor_1);
		temp_line_1 = new Line(T1, T2);

		//Druga moznost
		smerni_vektor_2 = calc_DP(l1->T2, l2->T1);
		T1 = new Point(l1->T1);
		T1->move_by(smerni_vektor_2);
		T2 = new Point(l1->T2);
		T2->move_by(smerni_vektor_2);
		temp_line_2 = new Line(T1, T2);

		diff_1 = distance_between_points(l2->get_center_of_line(), temp_line_1->get_center_of_line());
		diff_2 = distance_between_points(l2->get_center_of_line(), temp_line_2->get_center_of_line());
		if (diff_1 < diff_2)
		{
			T2 = new Point(l1->T2);
			T2->move_by(smerni_vektor_1);
			l2->T2->move_to(T2);
		}
		else
		{
			T2 = new Point(l1->T1);
			T2->move_by(smerni_vektor_2);
			l2->T2->move_to(T2);
		}
		break;
	case 2: //okrog T2
		//Prva moznost
		smerni_vektor_1 = calc_DP(l1->T1, l2->T2);
		T1 = new Point(l1->T1);
		T1->move_by(smerni_vektor_1);
		T2 = new Point(l1->T2);
		T2->move_by(smerni_vektor_1);
		temp_line_1 = new Line(T1, T2);

		//Druga moznost
		smerni_vektor_2 = calc_DP(l1->T2, l2->T2);
		T1 = new Point(l1->T1);
		T1->move_by(smerni_vektor_2);
		T2 = new Point(l1->T2);
		T2->move_by(smerni_vektor_2);
		temp_line_2 = new Line(T1, T2);

		diff_1 = distance_between_points(l2->get_center_of_line(), temp_line_1->get_center_of_line());
		diff_2 = distance_between_points(l2->get_center_of_line(), temp_line_2->get_center_of_line());
		if (diff_1 < diff_2)
		{
			T1 = new Point(l1->T2);
			T1->move_by(smerni_vektor_1);
			l2->T1->move_to(T1);
		}
		else
		{
			T1 = new Point(l1->T1);
			T1->move_by(smerni_vektor_2);
			l2->T1->move_to(T1);
		}
		break;
	}
	return 1;
}

int Constraint_Manager::make_line_equal_length(Line *l1, Line *l2, int mode)
{
	//mode:
	// 0 - sprememba iz sredine
	// 1 - T1 je fiksna
	// 2 - T2 je fiksna
	Point *Center,*t1,*t2, *smerni_vektor;
	float length1, length2;
	
	length1 = l1->get_length();
	length2 = l2->get_length();

	smerni_vektor = calc_DP(l2->T1, l2->T2);
	Center = new Point(l2->get_center_of_line());

	t1 = new Point();
	t2 = new Point();
	switch(mode)
	{
	case 0:
		t1->x = round_value(Center->x - length1/length2 * 1/2 * smerni_vektor->x);
		t1->y = round_value(Center->y - length1/length2 * 1/2 * smerni_vektor->y);
		t2->x = round_value(Center->x + length1/length2 * 1/2 * smerni_vektor->x);
		t2->y = round_value(Center->y + length1/length2 * 1/2 * smerni_vektor->y);
		l2->move_vertex_to(1,t1);
		l2->move_vertex_to(2,t2);
		break;
	case 1:
		t2->x = round_value(l2->T1->x + length1/length2 * smerni_vektor->x);
		t2->y = round_value(l2->T1->y + length1/length2 * smerni_vektor->y);
		l2->move_vertex_to(2,t2);
		break;
	case 2:
		t1->x = round_value(l2->T2->x - length1/length2 * smerni_vektor->x);
		t1->y = round_value(l2->T2->y - length1/length2 * smerni_vektor->y);
		l2->move_vertex_to(1,t1);
		break;
	}
	return 1;
}

int Constraint_Manager::connect_lines(Line *l1, Line *l2, int i, int mode, int mode2)
{
	//mode pove, katera crta bo fiksirana, tj. mode = 1 pomeni l1 je fiskirana
	//mode2 pa pove, kako bomo prestavili l2 (0 - sredinsko, 1 - tocko T1, 2 - tocko T2)
	Point *t1,*t2,*dp;
	switch(i)
	{
	case 1:
		t1 = new Point(l1->T1);
		t2 = new Point(l2->T1);
		break;
	case 2:
		t1 = new Point(l1->T1);
		t2 = new Point(l2->T2);
		break;
	case 3:
		t1 = new Point(l1->T2);
		t2 = new Point(l2->T1);
		break;
	case 4:
		t1 = new Point(l1->T2);
		t2 = new Point(l2->T2);
		break;
	}

	cout << "smo v connect lines, premikali bomo tocki: " << endl;
	printout_point(t1);
	printout_point(t2);
	cout << "ct je: " << i;

	switch(mode)//katera se ne premika
	{
	case 1: 
		dp = calc_DP(t2,t1);
		cout << ", za koliko prestavi: ";
		printout_point(dp);
		switch (mode2)
		{
		case 0:
			l2->move_object_by(dp);
			break;
		case 1:
			l2->move_vertex_by(1,dp);
			break;
		case 2:
			l2->move_vertex_by(2,dp);
			break;
		}
		break;
	case 2:
		dp = calc_DP(t1,t2);
		switch (mode2)
		{
		case 0:
			l1->move_object_by(dp);
			break;
		case 1:
			l1->move_vertex_by(1,dp);
			break;
		case 2:
			l1->move_vertex_by(2,dp);
			break;
		}
		break;
	}

	return 1;
}


int Constraint_Manager::check_for_constraint_type(Object * o, int type)
{
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;

	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);
		if (cn->get_constraint_type() == type)
			return 1;
	}
	return 0;
}


//preverjanje in dodajanje omejitev
void Constraint_Manager::add_and_check(list<Object*> *objects, list<Object*> *new_objects)
{
	list<Object*>::iterator iter_obj;
	list<Object*>::iterator iter_no;
	list<Object*>::iterator iter_nx;
	list<Object*>::iterator iter_plus;

	Object *object, *new_object, *next_object;
	Line *line, *new_line, *next_line;
	
	float epsilon_a = 10;//this->ai->get_angle_type_1_epsilon(); //za vertikalne in horizontalne crte
	float epsilon_c = 10;//this->ai->get_connection_type_1_epsilon(); //za stikanje
	float epsilon_p = 20; //za paralelnost, ko bo
	float epsilon_l = 20; //TODO: this->ai->get_length_epsilon(); //za dolzino
	int ct;
	
	//Za en sam nov objekt
	if (new_objects->size() == 1)
	{
		iter_no = new_objects->begin();
		new_object = (*iter_no);
		new_line = (Line*) new_object;
		
		cout << endl;

		if (is_line_horizontal(new_line, epsilon_a) == 1)
		{
			cout << "Horizontal!" << endl;
			add_constraint(new Horizontal(new_object));
		}
		else if (is_line_vertical(new_line, epsilon_a) == 1)
		{
			cout << "Vertical!" << endl;
			add_constraint(new Vertical(new_object));
		}

		for (iter_obj = objects->begin(); iter_obj != objects->end(); iter_obj++)
		{
			object = (*iter_obj);
			line = (Line*) object;

			if ((ct = are_lines_connected(line,new_line,epsilon_c)) > 0)
			{
				add_constraint(new Connection(object, new_object, ct));
			}
			if (are_lines_parallel(line,new_line,epsilon_p)==1)
			{
				if (!(check_for_constraint_type(new_object, 5) || check_for_constraint_type(new_object, 5)))
				{
					cout << "Parallel!" << endl;
					add_constraint(new Parallel(object, new_object));
				}
			}
			if (are_lines_equal_length(line,new_line, epsilon_l)==1)
			{
				add_constraint(new Length(object,new_object));
			}
		} //for
	}//if

	//Za vec novih objektov
	else
	{	
		for(iter_no = new_objects->begin(); iter_no != new_objects->end(); iter_no++)
		{
			new_object = (*iter_no);
			new_line = (Line*) new_object;

			if (is_line_horizontal(new_line, epsilon_a) == 1)
			{
				add_constraint(new Horizontal(new_object));
			}
			else if (is_line_vertical(new_line, epsilon_a) == 1)
			{
				add_constraint(new Vertical(new_object));
			}

			iter_plus = iter_no;
			iter_plus++;
 
			for (iter_nx = iter_plus; iter_nx != new_objects->end(); iter_nx++)
			{
				next_object = (*iter_nx);
				next_line = (Line*) next_object;

				if ((ct = are_lines_connected(new_line,next_line,epsilon_c)) > 0)
				{
					add_constraint(new Connection(new_object, next_object, ct));
				}
				if (are_lines_parallel(new_line,next_line,epsilon_l)==1)
				{
					add_constraint(new Parallel(new_object, next_object));
				}
				if (are_lines_equal_length(new_line,next_line,30)==1)
				{
					add_constraint(new Length(new_object, next_object));
				}
			}//for
		}//for
	} //else
}

int Constraint_Manager::check_constraints(Object *o)
{
	int OK = 0;
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;
	Connection *con;
	Line *line, *other_line;
	int ct;

	line = (Line*)o;
		
	/********************/
	//za horizontalo
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{	
		cn = (*iter_cn); 
		if (cn->get_constraint_type() == 4)
		{
			if (cn->get_u() == 0) 
			{
				cn->set_u(1);
				cout << "popravljamo na horizontalo!" << endl;
				make_line_horizontal(line);
			}
		}
	}
	/********************/
	//za vertikalo
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn); 
		if (cn->get_constraint_type() == 5)
		{
			if (cn->get_u() == 0) 
			{
				cn->set_u(1);
				cout << "popravljamo na vertikalo!" << endl;
				make_line_vertical(line);
			}
		}
	}
	/********************/
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{	
		cn = (*iter_cn); 
		if (cn->get_constraint_type() == 1)
		{
			if (cn->get_u() == 0) 
			{
				int m = 0;
				int n;
				other_line = cn->get_other_line(line);
				con = (Connection*) cn;
				ct = con->get_ct(other_line,line);
				if (other_line->get_age() < line->get_age()) 
				{
					cout << "popravljamo stikanje, ct :" << ct << endl;
					cout << "stik s crto: " << other_line->get_age() << endl;
					m = con->get_connected_vertex_number(line);

					if (m == 1)
					{
						n = check_for_connection_with_older_line(line,2);
						
					}
					else //m=2
					{
						n = check_for_connection_with_older_line(line,1);
					}
					switch(n)
					{
					case 0:
						connect_lines(other_line, line, ct ,1 ,0);
						break;
					case 1:
						connect_lines(other_line, line, ct, 1, 0);
						break;
					case 2:
						connect_lines(other_line, line, ct, 1, m);
						break;
					}
					cn->set_u(1);	
				}//if
			}//if
		}//if
	}//for
	/********************/
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{	
		cn = (*iter_cn); 
		if (cn->get_constraint_type() == 3)
		{
			if ((cn->get_u() == 0) && (cn->get_broken() != 1)) 
			{
				int c = 0;
				int d;
				other_line = cn->get_other_line(line);
				if (other_line->get_age() < line->get_age()) 
				{
					c = check_for_connection_with_older_line(line,1); 
					
					d = check_for_connection_with_older_line(line,2);
					
					switch(c)
					{
					case 0:
						switch(d)
						{
						case 0: //ni stikanj
							make_line_equal_length(other_line, line, 0);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje
							make_line_equal_length(other_line, line, 2);
							//cn->set_broken(1);
							break;
						case 2: //v tocki 2 je uveljavljeno stikanje
							make_line_equal_length(other_line, line, 2);
							break;
						}
						break;
					case 1: //v tocki 1 je neuveljavljeno stikanje
						switch(d)
						{
						case 0: //ni stikanja v tocki 2
							make_line_equal_length(other_line, line, 1);
							//cn->set_broken(1);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje
							make_line_equal_length(other_line, line, 2);
							//cn->set_broken(1);
							break;
						case 2: //v tocki 2 je uveljavljeno stikanje
							make_line_equal_length(other_line, line, 2);
							//cn->set_broken(1);
							break;
						}
						break;						
					case 2: //v tocki 1 je uveljavljeno stikanje
						switch(d)
						{
						case 0: //ni stikanja v tocki 2
							make_line_equal_length(other_line, line, 1);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje - ki se bo uveljavilo
							make_line_equal_length(other_line, line, 1);
							//cn->set_broken(1);
							break;	
						case 2: //v tocki 2 je uveljavljeno stikanje, zato ne moremo spremeniti
							//dolzine - zlom
							cout << "ZLOM LENGTHA!!!!" << endl;
							cn->set_broken(1);
							break;
						}
						break;
					}
				cn->set_u(1);
				} //if
			}//if
			OK *= cn->get_u();
		}
	}
	/********************/
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{	
		cn = (*iter_cn); 
		if (cn->get_constraint_type() == 2)
		{
			if ((cn->get_u() == 0) && (cn->get_broken() != 1)) 
			{
				int c = 0;
				int d;
				other_line = cn->get_other_line(line);
				if (other_line->get_age() < line->get_age()) 
				{
					c = check_for_connection_with_older_line(line,1); 
					
					d = check_for_connection_with_older_line(line,2);
					
					switch(c)
					{
					case 0:
						switch(d)
						{
						case 0: //ni stikanj
							make_lines_parallel(other_line, line, 0);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje
							make_lines_parallel(other_line, line, 2);
							//cn->set_broken(1);
							break;
						case 2: //v tocki 2 je uveljavljeno stikanje
							make_lines_parallel(other_line, line, 2);
							break;
						}
						break;
					case 1: //v tocki 1 je neuveljavljeno stikanje
						switch(d)
						{
						case 0: //ni stikanja v tocki 2
							make_lines_parallel(other_line, line, 1);
							//cn->set_broken(1);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje
							make_lines_parallel(other_line, line, 2);
							//cn->set_broken(1);
							break;
						case 2: //v tocki 2 je uveljavljeno stikanje
							make_lines_parallel(other_line, line, 2);
							//cn->set_broken(1);
							break;
						}
						break;						
					case 2: //v tocki 1 je uveljavljeno stikanje
						switch(d)
						{
						case 0: //ni stikanja v tocki 2
							make_lines_parallel(other_line, line, 1);
							break;
						case 1: //v tocki 2 je neuveljavljeno stikanje - ki se bo uveljavilo
							make_lines_parallel(other_line, line, 1);
							//cn->set_broken(1);
							break;	
						case 2: //v tocki 2 je uveljavljeno stikanje, zato ne moremo spremeniti
							cn->set_broken(1);
							break;
						}
						break;
					}
				cn->set_u(1);
				} //if
			}//if
			OK *= cn->get_u();
		}
	}	
	return 1;
}

void Constraint_Manager::recheck_objects(Object *o)
{
	Object *other;
	o->set_recheck(1);

	changed_objects->push_back(o);
	
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;

	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);
		if (cn->get_visited_on_move() == 0)
		{
			cn->set_visited_on_move(1);
			if (cn->get_constraint_type() == 1)
			{
				other = cn->get_other_object(o);
				if (other->get_recheck() == 0)
					recheck_objects(other);
			}
		}
	}
}


void Constraint_Manager::split_changed_objects()
{
	Object *o;
	list<Object*>::iterator iter_changed;
	list<Object*> *unchanged = new list<Object*>;
	
	new_objects->clear();
	unchanged_objects->clear();

	for (iter_changed = changed_objects->begin(); iter_changed != changed_objects->end(); iter_changed++)
	{
		o = (*iter_changed);
		if (o != NULL)
		{
			if (o->get_changed() == 1)
			{
				remove_constraints_from_line((Line*)o);
				o->set_changed(0);
				o->set_recheck(0);
				new_objects->push_back(o);
			}
			else
			{	
				o->set_recheck(0);
				unchanged_objects->push_back(o);
			}
		}
	}
	remove_broken_constraints();
}

void Constraint_Manager::recheck_constraints()
{
	list <Object*>::iterator iter_changed;
	Object *o;

	changed_objects->clear();
	for (iter_changed = new_objects->begin(); iter_changed != new_objects->end(); iter_changed++)
	{
		o = (*iter_changed);
		o->set_youngest();
		changed_objects->push_back(o);
		add_and_check(unchanged_objects,changed_objects);
		unchanged_objects->push_back(o);
		changed_objects->clear();
	}
	for (iter_changed = new_objects->begin(); iter_changed != new_objects->end(); iter_changed++)
	{
		o = (*iter_changed);
		check_constraints((Line*)o);
	}
	changed_objects->clear();
	new_objects->clear();
}





unsigned char Constraint_Manager::move_vertex_by(Object *o, unsigned char vertex, Point *dp) {
	move_object_vertex_by(o,vertex,dp);
	clear_visited_on_constraints();
	return 1;
}
//premikanje
int Constraint_Manager::move_object_vertex_by(Object *o, int vertex, Point *dp)
{
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;
	Connection *con;
	Object *other;

	o->set_changed(1);
	o->set_already_moved(1);

	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);

		if ((cn->get_constraint_type() == 1) && (cn->get_visited_on_move() != 1)) //ce najdemo stik, ki ga se nismo obiskali
		{
			con = (Connection*) cn;

			cn->set_visited_on_move(1); //oznacimo obisk
			
			if (con->get_connected_vertex_number(o) == vertex) //premaknili smo del stika
			{
				other = cn->get_other_object(o);
				if (other->get_already_moved() != 1)
				{
					move_object_vertex_by(other, con->get_connected_vertex_number(other),dp);
				}
			}
		}
	}
	o->set_already_moved(0);
	o->get_vertex_point(vertex)->move_by(dp);

	obj_changed = o;
	return 1;
}

unsigned char Constraint_Manager::move_object_by(Object *o, Point *dp) {
	move_object_whole_by(o,dp);
	clear_visited_on_constraints();
	return 1;
}

int Constraint_Manager::move_object_whole_by(Object *o, Point *dp)
{
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;
	Connection *con;
	Object *other;

	
	o->set_already_moved(1);
	
	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);

		if ((cn->get_constraint_type() == 1) && (cn->get_visited_on_move() != 1))
		{
			con = (Connection*) cn;
			cn->set_visited_on_move(1);
			if (cn->get_constraint_type() == 1)
			{
				other = cn->get_other_object(o);
				if (other->get_already_moved() != 1)
				{
					move_object_whole_by(other, dp);
				}
			}
		}
	}
	o->move_object_by(dp);
	o->set_already_moved(0);

	return 1;
}

//ostale funkcije, ki pomagajo pri delu z omejitvami
unsigned char Constraint_Manager::check_for_connection_with_older_line(Line *line, unsigned char vertex)
{
	list <Constraint*>::iterator iter_cn;

	//vraca:
	// 0 - v tocki vertex ni stikanja
	// 1 - v tocki vertex je stikanje, vendar se ni bilo uveljavljeno
	// 2 - v tocki vertex je stikanje, ki je tudi ze bilo uveljavljeno
	//TODO: popravi

	Constraint *cn;
	Connection *con;
	Line *other;
	

	for(iter_cn = constraint_list->begin(); iter_cn != constraint_list->end(); iter_cn++)
	{
		cn = (*iter_cn);
		
		if (cn->get_constraint_type() == 1)
		{
			con = (Connection*) cn;
			other = con->get_other_line(line);
			if (other->get_age() < line->get_age())
			{
				if (con->get_connected_vertex_number(line) == vertex)
				{
					if (con->get_u() == 0)
					{
						return 1; //ni uveljavljen constraint
					}
					else
						return 2;
				}
			}
		}
	}
	return 0;
}

void Constraint_Manager::remove_broken_constraints()
{
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;

	for (iter_cn = constraint_list->begin(); iter_cn != constraint_list->end(); iter_cn++)
	{
		cn = (*iter_cn);
		if (cn->get_broken() == 1)
		{
			delete_constraint(cn);
			iter_cn = constraint_list->begin();
		}
	}
}

void Constraint_Manager::set_connections_u(Line *line, int u)
{
}

void Constraint_Manager::remove_constraints_from_line(Line *line)
{
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;
		
	for (iter_cn = line->constraints.begin(); iter_cn != line->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);
		cn->set_broken(1);
	}
}


void Constraint_Manager::clear_visited_on_constraints()
{

	list <Constraint*>::iterator iter_cn;
	Constraint *cn;
	
	for (iter_cn = constraint_list->begin(); iter_cn != constraint_list->end(); iter_cn++)
	{
		cn = (*iter_cn);
		cn->set_visited_on_move(0);
	}
}

void Constraint_Manager::clear_flags_on_objects()
{
	list <Object*>::iterator iter_obj;
	Object *o;
	
	for (iter_obj = object_list->begin(); iter_obj != object_list->end(); iter_obj++)
	{
		o = (*iter_obj);
		o->set_changed(0);
		o->set_recheck(0);
	}
}

unsigned char Constraint_Manager::check_for_connection(Line *line, unsigned char vertex)
{
	return 0;
}

void Constraint_Manager::printout_point(Point *T)
{
	cout << "T: " << T->x << "," << T->y << endl;
}

void Constraint_Manager::printout_line(Line *l)
{
	cout << "T1 - ";
	printout_point(l->T1);
	cout << "T2 - ";
	printout_point(l->T2);
}

Point *Constraint_Manager::calc_DP(Point *Start, Point *End)
{
	return new Point(End->x - Start->x, End->y - Start->y);
}


double Constraint_Manager::distance_between_points(Point *T1, Point *T2)
{
	return sqrt(double((T1->x - T2->x)*(T1->x - T2->x) + (T1->y - T2->y)*(T1->y - T2->y)));
}

int Constraint_Manager::round_value(double value)
{
	if (value - floor(value) < 0.5) 
		return (int)floor(value);
	else
		return (int)ceil(value);
}

void Constraint_Manager::draw_objects(FXDCWindow *dc)
{
	for (iter_obj = object_list->begin(); iter_obj != object_list->end(); iter_obj++)
	{
		(*iter_obj)->draw(dc);
	}
}

void Constraint_Manager::draw_constraints(FXDCWindow *dc)
{
	for (iter_cn = constraint_list->begin(); iter_cn != constraint_list->end(); iter_cn++)
	{
		(*iter_cn)->draw(dc);
	}
}

void Constraint_Manager::straightener() {
	list <Constraint*>::iterator iter_cn, iter2;
	Constraint *cn,*cn2;
	Connection *con,*con2;
	Object *obj1, *obj2;
	int v1,v2, str;
	Point *T1,*T2;

	str = 0;

	for (iter_cn = constraint_list->begin(); iter_cn != constraint_list->end(); iter_cn++)
	{
		cn = (*iter_cn);

		if (cn->get_constraint_type() == 1)
		{
			con = (Connection*) cn;

			obj1 = cn->get_object(1);
			obj2 = cn->get_object(2);

			v1 = con->get_connected_vertex_number(obj1);
			v2 = con->get_connected_vertex_number(obj2);

			
			for (iter2 = obj1->constraints.begin(); iter2 != obj1->constraints.end(); iter2++)
			{
				cn2 = (*iter2);

				if (cn2->get_constraint_type() == 1)
				{
					con2 = (Connection*) cn2;

					if (cn->get_age() != cn2->get_age()) //ce nismo nasli isti stik
					{
						if (con2->get_connected_vertex_number(obj1) != v1) //preverimo, ce je v isti tocki se kaksen stik
						{
							str = 1; //ni stika v isti tocki
						}
						else
						{
							str = 0; //v isti tocki imamo se en stik
							break;
						}
					}
				}
			}

			if (str == 1)//lahko poravnamo
			{
				T1 = obj1->get_other_vertex_point(v1);
				T2 = obj2->get_other_vertex_point(v2);

				delete_object(obj1);
				delete_object(obj2);

				add_object(new Line(T1,T2));

				iter_cn = constraint_list->begin();
			}
		}
	}
}

void Constraint_Manager::group_connected_objects(list<Object*> *group, Object *o)
{
	Object *other;
	o->set_recheck(1);

	group->push_back(o);
	
	list <Constraint*>::iterator iter_cn;
	Constraint *cn;

	for (iter_cn = o->constraints.begin(); iter_cn != o->constraints.end(); iter_cn++)
	{
		cn = (*iter_cn);
		if (cn->get_visited_on_move() == 0)
		{
			cn->set_visited_on_move(1);
			if (cn->get_constraint_type() == 1)
			{
				other = cn->get_other_object(o);
				if (other->get_recheck() == 0)
					group_connected_objects(group,other);
			}
		}
	}
}

Object *Constraint_Manager::find_closest_to_connect(Object *o, float epsilon)
{
	list <Object*>::iterator iter_obj;
	Object *obj;

	for (iter_obj = object_list->begin(); iter_obj != object_list->end(); iter_obj++)
	{
		obj = (*iter_obj);
		if (are_lines_connected((Line*)obj, (Line*)o, epsilon) != 0)
			return obj;
	}
	return NULL;
}